//=============================================================================

#include "cinder/app/App.h"
#include "cinder/app/RendererGl.h"
#include "cinder/Color.h"
#include "cinder/gl/gl.h"
#include "cinder/Vector.h"
#include "Ship.h"

using namespace ci;
using namespace ci::app;
using namespace gl;
using namespace std;

//-----------------------------------------------------------------------------

class CinderApp : public App
{
public:
   static void prepareSettings(Settings* settings);
   void setup() override;
   void update() override;
   void draw() override;

   virtual void keyDown(KeyEvent event) override;
   virtual void keyUp(KeyEvent event) override;

private:
   bool mKeyRight=false, mKeyLeft=false, mKeyUp=false, mKeyDown=false;
   bool mKeyFire = false;

   ShipPtr mShip;
   vector<TorpedoPtr> mTorpedos;
};

//-----------------------------------------------------------------------------

void CinderApp::prepareSettings(Settings* settings)
{
   settings->setTitle("Cinder 10 - Torpedos initial integrieren");
   settings->setResizable(false);
   settings->setWindowPos(100, 100);
   settings->setWindowSize(640, 480);
   settings->setFrameRate(30.0f);
}

//-----------------------------------------------------------------------------

void CinderApp::setup()
{
   Ship::sScreenWidth = static_cast<float>(getWindowWidth());
   Ship::sScreenHeigth = static_cast<float>(getWindowHeight());
   Torpedo::sScreenWidth = Ship::sScreenWidth;
   Torpedo::sScreenHeigth = Ship::sScreenHeigth;

   mShip = make_shared<Ship>();
}

//-----------------------------------------------------------------------------

void CinderApp::update()
{
   if (mKeyUp && mShip)
   {
      mShip->accelerate(+1.f);
   }
   if (mKeyDown && mShip)
   {
      mShip->accelerate(-1.f);
   }
   if (mKeyRight && mShip)
   {
      mShip->rotate(+9.f);
   }
   if (mKeyLeft && mShip)
   {
      mShip->rotate(-9.f);
   }

   if (mKeyFire && mShip)
   {
      TorpedoPtr f = mShip->fire();
      mTorpedos.push_back(f);
   }

   if (mShip)
   {
      mShip->update();
   }
   for_each(begin(mTorpedos), end(mTorpedos), [](const TorpedoPtr& p) { p->update(); });
}

//-----------------------------------------------------------------------------

void CinderApp::draw()
{
   clear(Color(0, 0, 0));

   if (mShip)
   {
      mShip->draw();
   }
   for_each(cbegin(mTorpedos), cend(mTorpedos), [](const TorpedoPtr& p) { p->draw(); });
}

//-----------------------------------------------------------------------------

void CinderApp::keyDown(KeyEvent event)
{
   switch (event.getCode())
   {
   case KeyEvent::KEY_RIGHT:
      mKeyRight = true;
      break;
   case KeyEvent::KEY_LEFT:
      mKeyLeft = true;
      break;
   case KeyEvent::KEY_UP:
      mKeyUp = true;
      break;
   case KeyEvent::KEY_DOWN:
      mKeyDown = true;
      break;
   case KeyEvent::KEY_x:
      mKeyFire = true;
      break;
   }
}

//-----------------------------------------------------------------------------

void CinderApp::keyUp(KeyEvent event)
{
   switch (event.getCode())
   {
   case KeyEvent::KEY_RIGHT:
      mKeyRight = false;
      break;
   case KeyEvent::KEY_LEFT:
      mKeyLeft = false;
      break;
   case KeyEvent::KEY_UP:
      mKeyUp = false;
      break;
   case KeyEvent::KEY_DOWN:
      mKeyDown = false;
      break;
   case KeyEvent::KEY_x:
      mKeyFire = false;
      break;
   }
}

//-----------------------------------------------------------------------------

CINDER_APP(CinderApp, RendererGl(), &CinderApp::prepareSettings)
